@@ -25,16 +25,23 @@
 #
 # ===============================================================================
 
-
-
 #' Read .las or .laz files
 #'
 #' Reads .las or .laz files into an object of class \link[=LAS-class]{LAS}. If several files are read at
 #' once the returned LAS object is considered as one LAS file. The optional parameters enable the user
 #' to save a substantial amount of memory by choosing to load only the attributes or points of interest.
-#' The LAS formats 1.1 to 1.4 are supported. Point Data Record Format 0,1,2,3,5,6,7,8 are supported.
+#' LAS formats 1.1 to 1.4 are supported. Point Data Record Format 0,1,2,3,5,6,7,8 are supported.\cr\cr
+#' `readLAS` is the original function and always works. Using one of the `read*LAS` functions
+#' adds information to the returned object to register a point-cloud type. Registering the correct point
+#' type **may** improve the performance of some functions by enabling users to select an appropriate spatial index.
+#' See \link[=lidR-spatial-index]{spatial indexing}. Notice that by legacy and for backwards-compatibility reasons, 
+#' `readLAS()` and `readALSLAS()` are equivalent because lidR was originally designed for ALS and thus the 
+#' original function `readLAS()` was (supposedly) used for ALS. Reading a TLS dataset with `readLAS()` instead 
+#' of `readTLSLAS()` is perfectly valid and performs similarly to versions `<= 3.0.0`, with neither 
+#' performance degradation nor improvements.
 #'
-#' \strong{Select:} the 'select' argument specifies the data that will actually be loaded. For example,
+#'
+#' **Select:** the 'select' argument specifies the data that will actually be loaded. For example,
 #' 'xyzia' means that the x, y, and z coordinates, the intensity and the scan angle will be loaded.
 #' The supported entries are t - gpstime, a - scan angle, i - intensity, n - number of returns,
 #' r - return number, c - classification, s - synthetic flag, k - keypoint flag, w - withheld flag,
@@ -44,11 +51,11 @@
 #' Also numbers from 1 to 9 for the extra bytes data numbers 1 to 9. 0 enables all extra bytes to be
 #' loaded and '*' is the wildcard that enables everything to be loaded from the LAS file. \cr
 #' Note that x, y, z are implicit and always loaded. 'xyzia' is equivalent to 'ia'.\cr\cr
-#' \strong{Filter:} the 'filter' argument allows filtering of the point cloud while reading files.
+#' **Filter:** the 'filter' argument allows filtering of the point cloud while reading files.
 #' This is much more efficient than \link{filter_poi} in many ways. If the desired filters are known
 #' before reading the file, the internal filters should always be preferred. The available filters are
-#' those from \code{LASlib} and can be found by running the following command: \code{readLAS(filter = "-help")}.
-#' (see also \link[rlas:read.las]{rlas::read.las}). From \code{rlas} v1.3.6 the transformation commands
+#' those from `LASlib` and can be found by running the following command: `readLAS(filter = "-help")`.
+#' (see also \link[rlas:read.las]{rlas::read.las}). From `rlas` v1.3.6 the transformation commands
 #' can also be passed via the argument filter.
 #'
 #' @param files characters. Path(s) to one or several a file(s). Can also be a
@@ -56,7 +63,6 @@
 #' @param select character. Read only attributes of interest to save memory (see details).
 #' @param filter character. Read only points of interest to save memory (see details).
 #'
-#'
 #' @return A LAS object
 #'
 #' @export
@@ -70,6 +76,7 @@
 #'
 #' # Negation of attributes is also possible (all except intensity and angle)
 #' las = readLAS(LASfile, select = "* -i -a")
+#' @md
 readLAS = function(files, select = "*", filter = "")
 {
   if (filter == "-h" | filter == "-help")
@@ -133,8 +140,9 @@ readLAS.LAScluster = function(files, select = NULL, filter = NULL)
 
   buffer <- X <- Y <- NULL
 
-  las <- readLAS(files@files, files@select,files@filter)
+  las <- readLAS(files@files, files@select, files@filter)
   las@proj4string <- files@proj4string
+  las@index <- files@index
 
   las@data[, buffer := LIDRNOBUFFER]
 
@@ -277,5 +285,5 @@ streamLAS.character = function(x, ofile, select = "*", filter = "", filter_wkt =
     }
   }
 
-  return(LAS(data, header, check = TRUE))
+  return(LAS(data, header, check = TRUE, index = LIDRDEFAULTINDEX))
 }
